@@ -482,6 +482,59 @@ static void add_uris_from_dns(const char *filename, int lnr,
 }
 #endif /* HAVE_LDAP_DOMAIN2HOSTLIST */
 
+/* check that the file is not world readable */
+static void check_permissions(const char *filename, const char *keyword)
+{
+  struct stat sb;
+  /* get file status */
+  if (stat(filename, &sb))
+  {
+    log_log(LOG_ERR, "cannot stat() %s: %s", filename, strerror(errno));
+    exit(EXIT_FAILURE);
+  }
+  /* check permissions */
+  if ((sb.st_mode & 0007) != 0)
+  {
+    if (keyword != NULL)
+      log_log(LOG_ERR, "%s: file should not be world readable if %s is set",
+              filename, keyword);
+    else
+      log_log(LOG_ERR, "%s: file should not be world readable", filename);
+    exit(EXIT_FAILURE);
+  }
+}
+
+/* check whether the specified path is readable */
+static void check_readable(const char *filename, int lnr,
+                       const char *keyword, const char *path)
+{
+  if (access(path, R_OK) != 0)
+  {
+    log_log(LOG_ERR, "%s:%d: %s: error accessing %s: %s",
+            filename, lnr, keyword, path, strerror(errno));
+    exit(EXIT_FAILURE);
+  }
+}
+
+/* check whether the specified path is a directory */
+static void check_dir(const char *filename, int lnr,
+                      const char *keyword, const char *path)
+{
+  struct stat sb;
+  if (stat(path, &sb))
+  {
+    log_log(LOG_ERR, "%s:%d: %s: cannot stat() %s: %s",
+            filename, lnr, keyword, path, strerror(errno));
+    exit(EXIT_FAILURE);
+  }
+  if (!S_ISDIR(sb.st_mode))
+  {
+    log_log(LOG_ERR, "%s:%d: %s: %s is not a directory",
+            filename, lnr, keyword, path);
+    exit(EXIT_FAILURE);
+  }
+}
+
 static void handle_krb5_ccname(const char *filename, int lnr,
                                const char *keyword, char *line)
 {
@@ -504,12 +557,7 @@ static void handle_krb5_ccname(const char *filename, int lnr,
       (strncasecmp(ccname, "WRFILE:", sizeof("WRFILE:") - 1) == 0))
   {
     ccfile = strchr(ccname, ':') + 1;
-    if (access(ccfile, R_OK) != 0)
-    {
-      log_log(LOG_ERR, "%s:%d: error accessing %s: %s",
-              filename, lnr, ccfile, strerror(errno));
-      exit(EXIT_FAILURE);
-    }
+    check_readable(filename, lnr, keyword, ccfile);
   }
   /* set the environment variable (we have a memory leak if this option
      is set multiple times) */
@@ -1120,28 +1168,6 @@ static MUST_USE char *get_base_from_rootdse(void)
   return base;
 }
 
-/* check that the file is not world readable */
-static void check_permissions(const char *filename, const char *keyword)
-{
-  struct stat sb;
-  /* get file status */
-  if (stat(filename, &sb))
-  {
-    log_log(LOG_ERR, "cannot stat() %s: %s", filename, strerror(errno));
-    exit(EXIT_FAILURE);
-  }
-  /* check permissions */
-  if ((sb.st_mode & 0007) != 0)
-  {
-    if (keyword != NULL)
-      log_log(LOG_ERR, "%s: file should not be world readable if %s is set",
-              filename, keyword);
-    else
-      log_log(LOG_ERR, "%s: file should not be world readable", filename);
-    exit(EXIT_FAILURE);
-  }
-}
-
 /* set the configuration information to the defaults */
 static void cfg_defaults(struct ldap_config *cfg)
 {
@@ -1179,6 +1205,9 @@ static void cfg_defaults(struct ldap_config *cfg)
   cfg->scope = LDAP_SCOPE_SUBTREE;
   cfg->deref = LDAP_DEREF_NEVER;
   cfg->referrals = 1;
+#if defined(HAVE_LDAP_SASL_BIND) && defined(LDAP_SASL_SIMPLE)
+  cfg->pam_authc_ppolicy = 1;
+#endif
   cfg->bind_timelimit = 10;
   cfg->timelimit = LDAP_NO_LIMIT;
   cfg->idle_timelimit = 0;
@@ -1395,6 +1424,17 @@ static void cfg_read(const char *filename, struct ldap_config *cfg)
     {
       handle_map(filename, lnr, keyword, line);
     }
+    else if (strcasecmp(keyword, "pam_authc_ppolicy") == 0)
+    {
+#if defined(HAVE_LDAP_SASL_BIND) && defined(LDAP_SASL_SIMPLE)
+      cfg->pam_authc_ppolicy = get_boolean(filename, lnr, keyword, &line);
+      get_eol(filename, lnr, keyword, &line);
+#else
+      log_log(LOG_ERR, "%s:%d: value %s not supported on platform",
+              filename, lnr, value);
+      exit(EXIT_FAILURE);
+#endif
+    }
     /* timing/reconnect options */
     else if (strcasecmp(keyword, "bind_timelimit") == 0)
     {
@@ -1442,7 +1482,7 @@ static void cfg_read(const char *filename, struct ldap_config *cfg)
     {
       value = get_strdup(filename, lnr, keyword, &line);
       get_eol(filename, lnr, keyword, &line);
-      /* TODO: check that the path is valid */
+      check_dir(filename, lnr, token, value);
       log_log(LOG_DEBUG, "ldap_set_option(LDAP_OPT_X_TLS_CACERTDIR,\"%s\")",
               value);
       LDAP_SET_OPTION(NULL, LDAP_OPT_X_TLS_CACERTDIR, value);
@@ -1453,7 +1493,7 @@ static void cfg_read(const char *filename, struct ldap_config *cfg)
     {
       value = get_strdup(filename, lnr, keyword, &line);
       get_eol(filename, lnr, keyword, &line);
-      /* TODO: check that the path is valid */
+      check_readable(filename, lnr, keyword, value);
       log_log(LOG_DEBUG, "ldap_set_option(LDAP_OPT_X_TLS_CACERTFILE,\"%s\")",
               value);
       LDAP_SET_OPTION(NULL, LDAP_OPT_X_TLS_CACERTFILE, value);
@@ -1463,7 +1503,7 @@ static void cfg_read(const char *filename, struct ldap_config *cfg)
     {
       value = get_strdup(filename, lnr, keyword, &line);
       get_eol(filename, lnr, keyword, &line);
-      /* TODO: check that the path is valid */
+      check_readable(filename, lnr, keyword, value);
       log_log(LOG_DEBUG, "ldap_set_option(LDAP_OPT_X_TLS_RANDOM_FILE,\"%s\")",
               value);
       LDAP_SET_OPTION(NULL, LDAP_OPT_X_TLS_RANDOM_FILE, value);
@@ -1481,7 +1521,7 @@ static void cfg_read(const char *filename, struct ldap_config *cfg)
     {
       value = get_strdup(filename, lnr, keyword, &line);
       get_eol(filename, lnr, keyword, &line);
-      /* TODO: check that the path is valid */
+      check_readable(filename, lnr, keyword, value);
       log_log(LOG_DEBUG, "ldap_set_option(LDAP_OPT_X_TLS_CERTFILE,\"%s\")",
               value);
       LDAP_SET_OPTION(NULL, LDAP_OPT_X_TLS_CERTFILE, value);
@@ -1491,7 +1531,7 @@ static void cfg_read(const char *filename, struct ldap_config *cfg)
     {
       value = get_strdup(filename, lnr, keyword, &line);
       get_eol(filename, lnr, keyword, &line);
-      /* TODO: check that the path is valid */
+      check_readable(filename, lnr, keyword, value);
       log_log(LOG_DEBUG, "ldap_set_option(LDAP_OPT_X_TLS_KEYFILE,\"%s\")",
               value);
       LDAP_SET_OPTION(NULL, LDAP_OPT_X_TLS_KEYFILE, value);
@@ -1526,7 +1566,7 @@ static void cfg_read(const char *filename, struct ldap_config *cfg)
     }
     else if (strcasecmp(keyword, "nss_disable_enumeration") == 0)
     {
-      cfg->nss_getgrent_skipmembers = get_boolean(filename, lnr, keyword, &line);
+      cfg->nss_disable_enumeration = get_boolean(filename, lnr, keyword, &line);
       get_eol(filename, lnr, keyword, &line);
     }
     else if (strcasecmp(keyword, "validnames") == 0)
@@ -1744,6 +1784,9 @@ static void cfg_dump(void)
   LOG_ATTMAP(LM_SHADOW, shadow, shadowInactive);
   LOG_ATTMAP(LM_SHADOW, shadow, shadowExpire);
   LOG_ATTMAP(LM_SHADOW, shadow, shadowFlag);
+#if defined(HAVE_LDAP_SASL_BIND) && defined(LDAP_SASL_SIMPLE)
+  log_log(LOG_DEBUG, "CFG: pam_authc_ppolicy %s", print_boolean(nslcd_cfg->pam_authc_ppolicy));
+#endif
   log_log(LOG_DEBUG, "CFG: bind_timelimit %d", nslcd_cfg->bind_timelimit);
   log_log(LOG_DEBUG, "CFG: timelimit %d", nslcd_cfg->timelimit);
   log_log(LOG_DEBUG, "CFG: idle_timelimit %d", nslcd_cfg->idle_timelimit);
