@@ -5,7 +5,7 @@
 
    Copyright (C) 1997-2006 Luke Howard
    Copyright (C) 2006-2007 West Consulting
-   Copyright (C) 2006-2014 Arthur de Jong
+   Copyright (C) 2006-2015 Arthur de Jong
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -406,7 +406,64 @@ static int do_sasl_interact(LDAP UNUSED(*ld), unsigned UNUSED(flags),
   }
 
 #if defined(HAVE_LDAP_SASL_BIND) && defined(LDAP_SASL_SIMPLE)
-static void handle_ppasswd_controls(MYLDAP_SESSION *session, LDAP *ld, LDAPControl **ctrls)
+static void print_ppolicy_expiry(MYLDAP_SESSION *session, unsigned int sec)
+{
+  unsigned int days = 0;
+  unsigned int hours = 0;
+  unsigned int minutes = 0;
+  /* return this warning so PAM can present it to the user */
+  if (strlen(session->policy_message) != 0)
+    return;
+  if (sec > 24 * 3600)
+  {
+    days = sec / (24 * 3600);
+    sec -= days * 24 * 3600;
+  }
+  if (sec > 3600)
+  {
+    hours = sec / 3600;
+    sec -= (hours * 3600);
+  }
+  if (sec > 60)
+  {
+    minutes = sec / 60;
+    sec -= minutes * 60;
+  }
+  if (days > 1)
+    mysnprintf(session->policy_message, sizeof(session->policy_message),
+               "Password will expires in %u days", days);
+  else if (days > 0)
+    mysnprintf(session->policy_message, sizeof(session->policy_message),
+               "Password will expires in %u hours", hours + 24);
+  else if (hours > 1)
+  {
+    if (minutes > 1)
+      mysnprintf(session->policy_message, sizeof(session->policy_message),
+                 "Password will expires in %u hours and %u minutes",
+                 hours, minutes);
+    else
+      mysnprintf(session->policy_message, sizeof(session->policy_message),
+                 "Password will expires in %u hours", hours);
+  }
+  else if (hours > 0)
+    mysnprintf(session->policy_message, sizeof(session->policy_message),
+               "Password will expires in %u minutes", minutes + 60);
+  else if (minutes > 1)
+  {
+    if (sec > 1)
+      mysnprintf(session->policy_message, sizeof(session->policy_message),
+                 "Password will expires in %u minutes and %u seconds",
+                 minutes, sec);
+    else
+      mysnprintf(session->policy_message, sizeof(session->policy_message),
+                 "Password will expires in %u minutes", minutes);
+  }
+  else
+    mysnprintf(session->policy_message, sizeof(session->policy_message),
+               "Password will expires in %u seconds", sec);
+}
+
+static void handle_ppolicy_controls(MYLDAP_SESSION *session, LDAP *ld, LDAPControl **ctrls)
 {
   int i;
   int rc;
@@ -434,13 +491,7 @@ static void handle_ppasswd_controls(MYLDAP_SESSION *session, LDAP *ld, LDAPContr
       sec = atol(seconds);
       log_log(LOG_DEBUG, "got LDAP_CONTROL_PWEXPIRING (password will expire in %ld seconds)",
               sec);
-      /* return this warning to PAM */
-      if (session->policy_response == NSLCD_PAM_SUCCESS)
-      {
-        session->policy_response = NSLCD_PAM_NEW_AUTHTOK_REQD;
-        mysnprintf(session->policy_message, sizeof(session->policy_message),
-                   "password will expire in %ld seconds",  sec);
-      }
+      print_ppolicy_expiry(session, (unsigned int)sec);
     }
     else if (strcmp(ctrls[i]->ldctl_oid, LDAP_CONTROL_PASSWORDPOLICYRESPONSE) == 0)
     {
@@ -453,9 +504,8 @@ static void handle_ppasswd_controls(MYLDAP_SESSION *session, LDAP *ld, LDAPContr
       else
       {
         /* log returned control information */
-        if (error != PP_noError)
-          log_log(LOG_DEBUG, "got LDAP_CONTROL_PASSWORDPOLICYRESPONSE (%s)",
-                  ldap_passwordpolicy_err2txt(error));
+        log_log(LOG_DEBUG, "got LDAP_CONTROL_PASSWORDPOLICYRESPONSE (%s)",
+                ldap_passwordpolicy_err2txt(error));
         if (expire >= 0)
           log_log(LOG_DEBUG, "got LDAP_CONTROL_PASSWORDPOLICYRESPONSE (password will expire in %d seconds)",
                   expire);
@@ -467,7 +517,8 @@ static void handle_ppasswd_controls(MYLDAP_SESSION *session, LDAP *ld, LDAPContr
             ((session->policy_response == NSLCD_PAM_SUCCESS) ||
              (session->policy_response == NSLCD_PAM_NEW_AUTHTOK_REQD)))
         {
-          session->policy_response = NSLCD_PAM_AUTHTOK_EXPIRED;
+          /* this means that the password has expired and must be reset */
+          session->policy_response = NSLCD_PAM_NEW_AUTHTOK_REQD;
           mysnprintf(session->policy_message, sizeof(session->policy_message),
                      "%s", ldap_passwordpolicy_err2txt(error));
         }
@@ -475,6 +526,8 @@ static void handle_ppasswd_controls(MYLDAP_SESSION *session, LDAP *ld, LDAPContr
                  ((session->policy_response == NSLCD_PAM_SUCCESS) ||
                   (session->policy_response == NSLCD_PAM_NEW_AUTHTOK_REQD)))
         {
+          /* this means that the account is locked and the user cannot log
+             in (the bind probably failed already) */
           session->policy_response = NSLCD_PAM_ACCT_EXPIRED;
           mysnprintf(session->policy_message, sizeof(session->policy_message),
                      "%s", ldap_passwordpolicy_err2txt(error));
@@ -482,6 +535,8 @@ static void handle_ppasswd_controls(MYLDAP_SESSION *session, LDAP *ld, LDAPContr
         else if ((error == PP_changeAfterReset) &&
                  (session->policy_response == NSLCD_PAM_SUCCESS))
         {
+          /* this indicates that the password must be changed before the
+             user is allowed to perform any other operation */
           session->policy_response = NSLCD_PAM_NEW_AUTHTOK_REQD;
           mysnprintf(session->policy_message, sizeof(session->policy_message),
                      "%s", ldap_passwordpolicy_err2txt(error));
@@ -490,22 +545,22 @@ static void handle_ppasswd_controls(MYLDAP_SESSION *session, LDAP *ld, LDAPContr
                  ((session->policy_response == NSLCD_PAM_SUCCESS) ||
                   (session->policy_response == NSLCD_PAM_NEW_AUTHTOK_REQD)))
         {
+          /* any other error is assumed to mean that the operation failed */
           session->policy_response = NSLCD_PAM_PERM_DENIED;
           mysnprintf(session->policy_message, sizeof(session->policy_message),
                      "%s", ldap_passwordpolicy_err2txt(error));
         }
-        else if ((expire >= 0) &&
-                 ((session->policy_response == NSLCD_PAM_SUCCESS) ||
-                  (session->policy_response == NSLCD_PAM_NEW_AUTHTOK_REQD)))
+        /* both expire and grace should just be warnings to the user */
+        if ((expire >= 0) && (strlen(session->policy_message) == 0))
         {
-          session->policy_response = NSLCD_PAM_NEW_AUTHTOK_REQD;
-          mysnprintf(session->policy_message, sizeof(session->policy_message),
-                     "Password will expire in %d seconds", expire);
+          /* if no other error has happened, this indicates that the password
+             will soon expire (number of seconds) */
+          print_ppolicy_expiry(session, (unsigned int)expire);
         }
-        else if ((grace >= 0) &&
-                 (session->policy_response == NSLCD_PAM_SUCCESS))
+        else if ((grace >= 0) && (strlen(session->policy_message) == 0))
         {
-          session->policy_response = NSLCD_PAM_NEW_AUTHTOK_REQD;
+          /* this indicates the number of grace logins that are left before
+             no further login attempts will be allowed */
           mysnprintf(session->policy_message, sizeof(session->policy_message),
                      "Password expired, %d grace logins left", grace);
         }
@@ -525,19 +580,25 @@ static int do_ppolicy_bind(MYLDAP_SESSION *session, LDAP *ld, const char *uri)
   int msgid;
   struct timeval timeout;
   LDAPMessage *result;
-  /* build password policy request control */
-  passwd_policy_req.ldctl_oid = LDAP_CONTROL_PASSWORDPOLICYREQUEST;
-  passwd_policy_req.ldctl_value.bv_val = NULL; /* none */
-  passwd_policy_req.ldctl_value.bv_len = 0;
-  passwd_policy_req.ldctl_iscritical = 0; /* not critical */
-  requestctrls[0] = &passwd_policy_req;
+  /* build policy request if pam_authc_ppolicy is set */
+  if (nslcd_cfg->pam_authc_ppolicy)
+  {
+    passwd_policy_req.ldctl_oid = LDAP_CONTROL_PASSWORDPOLICYREQUEST;
+    passwd_policy_req.ldctl_value.bv_val = NULL; /* none */
+    passwd_policy_req.ldctl_value.bv_len = 0;
+    passwd_policy_req.ldctl_iscritical = 0; /* not critical */
+    requestctrls[0] = &passwd_policy_req;
+  }
+  else
+    requestctrls[0] = NULL;
   requestctrls[1] = NULL;
   /* build password berval */
   cred.bv_val = (char *)session->bindpw;
   cred.bv_len = strlen(session->bindpw);
   /* do a SASL simple bind with the binddn and bindpw */
-  log_log(LOG_DEBUG, "ldap_sasl_bind(\"%s\",%s) (uri=\"%s\")", session->binddn,
-          (session->bindpw[0] != '\0') ? "\"***\"" : "\"\"", uri);
+  log_log(LOG_DEBUG, "ldap_sasl_bind(\"%s\",%s) (uri=\"%s\") (ppolicy=%s)",
+          session->binddn, (session->bindpw[0] != '\0') ? "\"***\"" : "\"\"",
+          uri, (requestctrls[0] == NULL) ? "no" : "yes");
   rc = ldap_sasl_bind(ld, session->binddn, LDAP_SASL_SIMPLE, &cred, requestctrls, NULL, &msgid);
   if (rc != LDAP_SUCCESS)
     return rc;
@@ -567,21 +628,25 @@ static int do_ppolicy_bind(MYLDAP_SESSION *session, LDAP *ld, const char *uri)
       ldap_msgfree(result);
     return LDAP_TIMEOUT;
   }
-  /* parse the result from the bind operation (frees result, gets controls) */
   responsectrls = NULL;
-  parserc = ldap_parse_result(ld, result, &rc, NULL, NULL, NULL, &responsectrls, 1);
-  if (parserc != LDAP_SUCCESS)
+  /* ignore any response controls unless we're interested in ppolicy */
+  if (nslcd_cfg->pam_authc_ppolicy)
   {
-    myldap_err(LOG_ERR, ld, parserc, "ldap_parse_result() failed");
+    /* parse the result from the bind operation (frees result, gets controls) */
+    parserc = ldap_parse_result(ld, result, &rc, NULL, NULL, NULL, &responsectrls, 1);
+    if (parserc != LDAP_SUCCESS)
+    {
+      myldap_err(LOG_ERR, ld, parserc, "ldap_parse_result() failed");
+      if (responsectrls != NULL)
+        ldap_controls_free(responsectrls);
+      return parserc;
+    }
+    /* handle any returned controls */
     if (responsectrls != NULL)
+    {
+      handle_ppolicy_controls(session, ld, responsectrls);
       ldap_controls_free(responsectrls);
-    return parserc;
-  }
-  /* handle any returned controls */
-  if (responsectrls != NULL)
-  {
-    handle_ppasswd_controls(session, ld, responsectrls);
-    ldap_controls_free(responsectrls);
+    }
   }
   /* return the result of the BIND operation */
   if (rc != LDAP_SUCCESS)
@@ -1094,6 +1159,7 @@ void myldap_get_policy_response(MYLDAP_SESSION *session, int *response,
     *message = session->policy_message;
 }
 
+/* perform a search operation, the connection is assumed to be open */
 static int do_try_search(MYLDAP_SEARCH *search)
 {
   int ctrlidx = 0;
@@ -1105,10 +1171,6 @@ static int do_try_search(MYLDAP_SEARCH *search)
   char *deref_attrs[2];
 #endif /* HAVE_LDAP_CREATE_DEREF_CONTROL */
   int msgid;
-  /* ensure that we have an open connection */
-  rc = do_open(search->session);
-  if (rc != LDAP_SUCCESS)
-    return rc;
   /* if we're using paging, build a page control */
   if ((nslcd_cfg->pagesize > 0) && (search->scope != LDAP_SCOPE_BASE))
   {
@@ -1270,7 +1332,10 @@ static int do_retry_search(MYLDAP_SEARCH *search)
       {
         /* try to start the search */
         pthread_mutex_unlock(&uris_mutex);
-        rc = do_try_search(search);
+        /* ensure that we have an open connection and start a search */
+        rc = do_open(search->session);
+        if (rc == LDAP_SUCCESS)
+          rc = do_try_search(search);
         if (rc == LDAP_SUCCESS)
         {
           pthread_mutex_lock(&uris_mutex);
@@ -1641,7 +1706,10 @@ MYLDAP_ENTRY *myldap_get_entry(MYLDAP_SEARCH *search, int *rcp)
         /* close connection on some connection problems */
         if ((rc == LDAP_UNAVAILABLE) || (rc == LDAP_SERVER_DOWN) ||
             (rc == LDAP_SUCCESS) || (rc == LDAP_TIMELIMIT_EXCEEDED) ||
-            (rc == LDAP_OPERATIONS_ERROR) || (rc == LDAP_PROTOCOL_ERROR))
+            (rc == LDAP_OPERATIONS_ERROR) || (rc == LDAP_PROTOCOL_ERROR) ||
+            (rc == LDAP_BUSY) || (rc == LDAP_UNWILLING_TO_PERFORM) ||
+            (rc == LDAP_TIMEOUT) || (rc == LDAP_CONNECT_ERROR) ||
+            (rc == LDAP_NOT_SUPPORTED))
         {
           do_close(search->session);
           /* retry once if no data has been received yet */
