@@ -3,6 +3,7 @@
 
    Copyright (C) 2009 Howard Chu
    Copyright (C) 2009-2014 Arthur de Jong
+   Copyright (C) 2015 Nokia Solutions and Networks
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -148,7 +149,7 @@ static void update_username(MYLDAP_ENTRY *entry, char *username,
     return;
   }
   /* check if the username is different and update it if needed */
-  if (strcmp(username, value) != 0)
+  if (STR_CMP(username, value) != 0)
   {
     log_log(LOG_INFO, "username changed from \"%s\" to \"%s\"",
             username, value);
@@ -711,6 +712,16 @@ static int try_pwmod(MYLDAP_SESSION *oldsession,
     rc = myldap_passwd(session, userdn, oldpassword, newpassword);
     if (rc == LDAP_SUCCESS)
     {
+      /* if user modifies own password, update credentials for the session */
+      if (binddn == userdn)
+        if (myldap_set_credentials(session, binddn, newpassword))
+        {
+          rc = LDAP_LOCAL_ERROR;
+          log_log(LOG_WARNING, "%s: shadowLastChange: modification failed: %s",
+                  userdn, ldap_err2string(rc));
+          myldap_session_close(session);
+          return rc;
+        }
       /* try to update the shadowLastChange attribute */
       if (update_lastchange(session, userdn) != LDAP_SUCCESS)
         /* retry with the normal session */
