@@ -2,7 +2,7 @@
 
 # pynslcd.py - main daemon module
 #
-# Copyright (C) 2010-2015 Arthur de Jong
+# Copyright (C) 2010-2016 Arthur de Jong
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -91,7 +91,7 @@ def display_version(fp):
     fp.write('%(PACKAGE_STRING)s\n'
              'Written by Arthur de Jong.\n'
              '\n'
-             'Copyright (C) 2010-2015 Arthur de Jong\n'
+             'Copyright (C) 2010-2016 Arthur de Jong\n'
              'This is free software; see the source for copying conditions.  There is NO\n'
              'warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n'
              % {'PACKAGE_STRING': constants.PACKAGE_STRING, })
@@ -311,8 +311,8 @@ if __name__ == '__main__':
         ctx = daemon.DaemonContext(
                       pidfile=pidfile,
                       signal_map={
-                          signal.SIGTERM: 'terminate',
-                          signal.SIGINT: 'terminate',
+                          signal.SIGTERM: u'terminate',
+                          signal.SIGINT: u'terminate',
                           signal.SIGPIPE: None,
                       })
     # start daemon
